﻿Imports System.Speech.Synthesis

Public Class ctrlSpeech

	' benötigte API-Deklarationen
	Private Declare Function SendMessage Lib "user32" Alias "SendMessageA" _
		(ByVal hwnd As IntPtr,
		ByVal wMsg As Integer,
		ByVal wParam As Integer,
		ByVal lParam As IntPtr) As Integer

	'Constants for Audio control
	Private Const WM_APPCOMMAND As UInteger = &H319
	Private Const APPCOMMAND_VOLUME_UP As UInteger = &HA
	Private Const APPCOMMAND_VOLUME_DOWN As UInteger = &H9
	Private Const APPCOMMAND_VOLUME_MUTE As UInteger = &H8

	Public syn As New SpeechSynthesizer
	Dim str As String, vspRate As Short
	Dim setColor As Color = Color.Lime

	Public Property TextToSpeech As String 'der zu sprechende Text
		Get
			Return vspRate
		End Get
		Set(ByVal Value As String)
			str = Value
		End Set
	End Property

	Public Property spRate As Short  'Geschwindigkeit
		Get
			Return vspRate
		End Get
		Set(ByVal Value As Short)
			vspRate = Value
		End Set
	End Property

	Public Sub New()
		InitializeComponent()
		With syn 'Standards für Synthesizer setzen
			.SetOutputToDefaultAudioDevice()
			.Rate = spRate        ' Geschwindigkeit: -10 bis +10
			.SelectVoiceByHints(VoiceGender.Female)
			AddHandler .SpeakProgress, AddressOf synProgress
		End With
		With voices 'die ComboBox bleibt unsichtbar
			' alle verfügbaren Stimmen auflisten
			For Each voice As InstalledVoice In syn.GetInstalledVoices
				.Items.Add(voice.VoiceInfo.Name)
			Next
			.SelectedIndex = 0
		End With
	End Sub

	Private Sub voices_SelectedIndexChanged(sender As Object, e As EventArgs) _
		Handles voices.SelectedIndexChanged
		'die Stimme für Sythesizer setzen
		syn.SelectVoice(voices.SelectedItem)
	End Sub

	Private Sub pMinus_Click(sender As Object, e As EventArgs) Handles pMinus.Click
		'Lautstärke verringern
		SendMessage(Handle, WM_APPCOMMAND, &H30292, APPCOMMAND_VOLUME_DOWN * &H10000)
	End Sub

	Private Sub pPlus_Click(sender As Object, e As EventArgs) Handles pPlus.Click
		'Lautstärke erhöhen
		SendMessage(Handle, WM_APPCOMMAND, &H30292, APPCOMMAND_VOLUME_UP * &H10000)
	End Sub

	Private Sub pStart_MouseDown(sender As Object, e As MouseEventArgs) _
		Handles pStart.MouseDown, pPlus.MouseDown, pMinus.MouseDown, pStopp.MouseDown,
		pMute.MouseDown, pPause.MouseDown
		'Button mit Rand und Hintergrundfarbe zeichnen
		Dim theButton As PictureBox = DirectCast(sender, PictureBox)
		setButtonPic(theButton)
	End Sub

	Private Sub pStart_MouseUp(sender As Object, e As MouseEventArgs) _
		Handles pStart.MouseUp, pPlus.MouseUp, pMinus.MouseUp, pStopp.MouseUp,
		pMute.MouseUp, pPause.MouseUp
		'Button mit ursprünglichem Erscheinungsbild und Aktion
		Dim theButton As PictureBox = DirectCast(sender, PictureBox)
		Select Case theButton.Name
			Case "pStart"
				'sprechen starten oder bei Pause fortsetzen
				If syn.State = SynthesizerState.Paused Then
					syn.Resume()
				Else
					syn.SpeakAsync(str)
				End If
				lblState.Text = "play"
			Case "pStopp"
				'sprechen stoppen
				syn.SpeakAsyncCancelAll()
				lblState.Text = "stopp"
			Case "pMute"
				'stumm schalten
				SendMessage(Handle, WM_APPCOMMAND, &H200EB0, APPCOMMAND_VOLUME_MUTE * &H10000)
				lblState.Text = "mute"
			Case "pPause"
				'pausieren des Sprechens
				syn.Pause()
				lblState.Text = "pause"
		End Select
		setButtonPic(theButton, False)
	End Sub

	Private Sub pStopp_MouseHover(sender As Object, e As EventArgs) _
		Handles pStart.MouseHover, pPlus.MouseHover, pMinus.MouseHover, pStopp.MouseHover,
		pMute.MouseHover, pPause.MouseHover
		sender.BorderStyle = BorderStyle.FixedSingle
	End Sub

	Private Sub pStopp_MouseLeave(sender As Object, e As EventArgs) _
		Handles pStart.MouseLeave, pPlus.MouseLeave, pMinus.MouseLeave, pStopp.MouseLeave,
		pMute.MouseLeave, pPause.MouseLeave
		sender.BorderStyle = BorderStyle.None
	End Sub

	Private Sub synProgress(sender As Object, e As SpeakProgressEventArgs)
		'Position des 'Lesezeigers' (wortorientiert)
		Debug.Print("CharPos: {0}" + Chr(9) + Chr(9) + "CharCount: {1}" _
						+ Chr(9) + Chr(9) + "AudioPos: {2}" + Chr(9) + """{3}""",
						e.CharacterPosition, e.CharacterCount, e.AudioPosition, e.Text)
	End Sub

	Private Sub setButtonPic(but As PictureBox, Optional art As Boolean = True)
		'Button Erscheinungsbild ändern
		With but
			If art Then 'gedrückt
				.BorderStyle = BorderStyle.FixedSingle
				.BackColor = setColor
			Else  'nicht gedrückt
				.BorderStyle = BorderStyle.None
				.BackColor = Color.Transparent
			End If
		End With
	End Sub

End Class
